<?php
//////////////////////////////////////////////////////////////////////////////////
//                         copyright (c) Arfooo Annuaire                          //
//                    by Hocine Guillaume (c) 2007 - 2008                        //
//                           http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class Admin_CategoryController extends Controller
{
    /**
    * set access privileges
    */
    function init()
    {
        $this->acl->allow('administrator', $this->name, "*");

        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect(FrontController::getRewrittedUrl("/admin/main/logIn"));
        }
    }

    /**
    * Display category data (subcategories, sites inside, forms to add site in this category or set ads)
    */
    function indexAction($parentCategoryId = 0, $page = 1)
    {
        //if isn't root category = 0
        if($parentCategoryId)
        {
            $perPage = 50;

            //get category data
            $currentCategory = $this->category->findByPk($parentCategoryId);

            //if this category doesn't exists'
            if(!$currentCategory)
            {
                return $this->return404();
            }

            //set category data
            $this->set("currentCategory", $currentCategory);

            //get validated sites inside this category
            $this->set("sites", $this->site->getValidatedSitesInCategory($parentCategoryId, $page, $perPage));

            //prepare pagination data
            $c = new Criteria();
            $c->add("categoryId", $parentCategoryId);
            $c->add("status", "validated");

            $totalPages = ceil($this->site->getCount($c) / $perPage);

            $this->set("pageNavigation", array("baseLink" => "/admin/category/index/$parentCategoryId/",
                                               "totalPages" => $totalPages,
                                               "currentPage" => $page));
                                               
            $c = new Criteria();
            $c->add("role", "webmaster");
            $this->set("webmasters", $this->user->generateList($c, "email"));

        }

        //set this category id
        $this->set("parentCategoryId", $parentCategoryId);

        //get category parents path
        $this->set("categoryParentsData", $this->categoryParent->getParentsData($parentCategoryId));

        //get select menu for categories
        $this->set("categoriesSelect", $this->category->createOptionsList());

        //set data for site form
        $this->set("allKeywordsList", $this->keyword->generateSortedList());
        $this->set("yesNoOptions", array("0" => _t("No"), "1" => _t("Yes")));
        $this->set("priorites", range(0, 10));
        $this->set("adCriterias", $this->adCriteria->getSelectList());

        //get subcategories in this category
        $this->set("categories", $this->category->getChilds($parentCategoryId));

        require(CODE_ROOT_DIR."config/flags.php");
        $this->set("countryFlags", $countryFlags);
    }

    /**
    * Get category data to edit and return to ajax query
    */
    function getCategoryDataAction()
    {
        $siteId = $this->request->categoryId;
        $site = $this->category->findByPk($siteId);
        $this->set($site->toArray());
        $this->viewClass = "JsonView";
    }

    /**
    * Display category edit form
    */
    function editAction($categoryId)
    {
        $this->set("categoryId", $categoryId);
        $this->set("categoriesSelect", $this->category->createOptionsList());
    }

    /**
    * Delete category
    */
    function deleteAction($categoryId)
    {
        $category = $this->category->findByPk($categoryId);

        if(empty($category))
        {
            return $this->return404();
        }

        $redirect = FrontController::getRewrittedUrl("/admin/category/index/".$category->parentCategoryId);

        //delete category with everything what is connected with it
        $this->category->delByPk($categoryId);

        $this->redirect($redirect);
    }

    /**
    * Handle edit form submission
    */
    function saveEditAction($categoryId)
    {
        $this->categoryParent->rebuildTable();

        //handle imageUpload
        $file = new UploadedFile("categoryImage");
        $file->addFilter("extension", array("jpg", "gif", "png"));

        //check image and save in upload location
        $imageSrc = $file->save(CODE_ROOT_DIR.'uploads/images_categories/');

        //create and save new category
        $fields = $this->request->getArray(array("name",
                                                 "title",
                                                 "description",
                                                 "possibleTender",
                                                 "forbidden"));

        $category = $this->category->findByPk($categoryId);
        $category->fromArray($fields);
        $rebuild = false;

        //handle categoryParentId change
        if($category->parentCategoryId != $this->request->parentCategoryId
        && $category->changeParent($this->request->parentCategoryId))
        {
            $rebuild = true;
        }

        //if image is uploaded and valida update name in category record
        if($imageSrc)
        {
            $category->imageSrc = $imageSrc;
        }

        $category->save();

        if($rebuild)
        {
            $this->categoryParent->rebuildTable();
        }

        $redirect = FrontController::getRewrittedUrl("/admin/category/index/".$category->parentCategoryId);
        $this->redirect($redirect);
    }

    /**
    * Save new category
    */
    function saveNewAction()
    {
        //handle image upload
        $file = new UploadedFile("categoryImage");
        $file->addFilter("extension", array("jpg", "gif", "png"));

        //check and save image
        $imageSrc = $file->save(CODE_ROOT_DIR.'uploads/images_categories/');

        //if no image or not valid
        if($imageSrc == false)
        {
            $imageSrc = 'defaultCategoryImage.gif';
        }

        //create and save new category
        $fields = $this->request->getArray(array("name",
                                                 "title",
                                                 "description",
                                                 "possibleTender",
                                                 "parentCategoryId",
                                                 "forbidden"));

        //create category record
        $newCategory = new CategoryRecord($fields);
        $newCategory->imageSrc = $imageSrc;
        $newCategory->save();

        //add node to categoryparents tree between all category parents
        $this->categoryParent->addNode($this->request->parentCategoryId, $newCategory->categoryId);

        $this->redirect($this->request->getRefererUrl());
    }
}

?>