<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class CategoryModel extends Model
{   
    protected $primaryKey = "categoryId";
    protected $dbTable = "categories";
    
    function getChilds($parentCategoryId)
    {
        $c = new Criteria();
        $c->addOrder("name");
        $c->add("parentCategoryId", $parentCategoryId); 
        
        return $this->findAll($c);
    }
    
    function createOptionsList($onlyPossibleTender = false)
    {
        $categories = Model::factoryInstance("categoryParent")->getCategoriesForSelect();
        $tree = new NavigationTree();
        
        foreach($categories as $category)
        {
            $value = array("name" => $category->name);
            $tree->addNode($category->categoryId, $category->parentId, $value);
        }
        
        $results = $tree->getFullOptionList(); 
        
        if($onlyPossibleTender)
        {
            $categoryPossible = $this->category->generateList(null, "possibleTender");
            
            foreach($results as $key => $result)
            {
                if(!$categoryPossible[$key])unset($results[$key]);
            }
        }
        
        asort($results);
        return $results;
    }
    
    function updateValidatedSitesCount()
    {
        $prefix = Config::get("DB_PREFIX");
        
        $sql = "UPDATE ".$prefix."categories SET validatedSitesCount = ( SELECT count( * ) 
                FROM ".$prefix."sites, ".$prefix."categoryparents 
                WHERE status = 'validated'  
                AND ".$prefix."categoryparents.parentId = ".$prefix."categories.categoryId 
                AND ".$prefix."categoryparents.categoryId = ".$prefix."sites.categoryId)";
                
        $this->db->sqlQuery($sql);
    }
    
    function del(Criteria $c)
    {
        $condition = $c->getCondition("categoryId");
        $categoryId = $condition['value'];

        $categories = $this->categoryParent->getAllChildCategoriesIds($categoryId);
        $categoriesIds = array();
        
        foreach($categories as $category)
        {
            $categoriesIds[] = $category['categoryId'];
        }

        //creating list of site to delete
        $c = new Criteria();
        $c->add("categoryId", $categoriesIds, "IN");
        $c->add("status", "banned", "!=");
        
        $siteIds = array();
        
        $sites = $this->site->findAll($c, "siteId");

        foreach($sites as $site)
        {
            $siteIds[] = $site['siteId'];
        }

        //delete all categories and subcategoeis   
        foreach($categoriesIds as $childCategoryId)
        {
            $c = new Criteria();
            $c->add("categoryId", $childCategoryId);
            parent::del($c);
            
            $c = new Criteria();
            $c->add("categoryId", $childCategoryId);
            $c->addOr("parentId", $childCategoryId);
            $this->categoryParent->del($c);
        }

        //deleting sites
        foreach($siteIds as $siteId)
        {
            $c = new Criteria();
            $c->add("siteId", $siteId);
            $this->site->del($c, false);
        }
        
        $this->updateValidatedSitesCount();
        Cacher::getInstance()->clean("tag", array("category", "site"));
    }
    
    function insert($data)
    {
        parent::insert($data);
        Cacher::getInstance()->clean("tag", array("category"));
    }
    
    function update($data, Criteria $c)
    {
        parent::update($data, $c);
        $this->updateValidatedSitesCount();
        Cacher::getInstance()->clean("tag", array("category"));       
    }

}

class CategoryRecord extends ModelRecord
{
    function getParentsData()
    {
        return Model::factoryInstance("categoryParent")->getParentsData($this->categoryId);
    }   
    
    function changeParent($newParentId)
    {
        $c = new Criteria();
        if(Model::factoryInstance("categoryParent")->isParentChild($this->categoryId, $newParentId))return false;
        
        $this->parentCategoryId = $newParentId;
        return true;
    }
}

?>