<?php
//////////////////////////////////////////////////////////////////////////////////
//                         copyright (c) Arfooo Annuaire                          //
//                    by Hocine Guillaume (c) 2007 - 2008                        //
//                           http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class SiteModel extends Model
{
    protected $primaryKey = "siteId";

    public function getIfHostIsBanned($hostname, $exceptSiteId = 0)
    {
        $hostname = addslashes($hostname);
        $c = new Criteria();
        $c->add("status", "banned");
        $c->add("siteId", $exceptSiteId);
        $c->add("LOCATE( '$hostname', `url` ) = 1 ");
        return $this->find($c) ? true : false;
    }

    public function getCountOfSiteCopiesInDifferentCategories($url)
    {
        $hostname = FrontController::getHostNameFromUrl($url);
        $hostname = addslashes($hostname);
        $c = new Criteria();
        $c->add("LOCATE( '$hostname', `url` ) = 1 ");

        return $this->getCount($c);
    }

    public function whetherSiteIsRegisteredInCategory($url, $categoryId)
    {
        $hostname = addslashes(FrontController::getHostNameFromUrl($url));

        $c = new Criteria();
        $c->add("categoryId", $categoryId);
        $c->add("LOCATE( '$hostname', `url` ) = 1 ");

        return ($this->getCount($c) > 0);
    }

    function selectWithNewFlag(Criteria $c, $fields = "*")
    {
        $packages = $this->package->generateList(null, "name, imageSrc", null, true);
        
        $noveltyPeriod = intval(Config::get("siteNoveltyPeriod"));
        $sites = $this->findAll($c, $fields.", packageId, ADDDATE(creationDate, $noveltyPeriod) > NOW() as isNew", true);
                 
        foreach($sites as $site)
        {
            if(isset($packages[$site->packageId]))
            {
                $site->packageImageSrc = $packages[$site->packageId]['imageSrc'];
                $site->packageName = $packages[$site->packageId]['name']; 
            }
            else
            {
                $site->packageImageSrc =  "";
                $site->packageName =  "";
            }
            
            $site->updateImageSrc();
        }

        return $sites;
    }

    function getWaitingSites($siteId = false, $page = false, $perPage = false)
    {
        $sites = array();
        $c = new Criteria();
        $c->add("status", "waiting");
        
        if(Config::get("emailConfirmationEnabled") && !Config::get("registrationOfWebmastersEnabled"))
        {
            $c->addOrder("emailConfirmed = '1' DESC");
        }
        
        $c->addOrder("siteType = 'premium' DESC");
        $c->addOrder("returnBond = '' ASC");
        $c->addOrder("creationDate");

        if($siteId)
        {
            $c->add("siteId", $siteId, is_array($siteId) ? "IN" : "=");
        }

        if($page !== false)$this->addPageCriteria($c, $page, $perPage);

        $c->setCalcFoundRows(true);
        $sites = $this->selectWithNewFlag($c);
        $this->saveFoundRowsCount();

        foreach($sites as $site)
        {
            $site->categoryParentsData = $this->categoryParent->getParentsData($site->categoryId);
            $site->keywords = $site->getKeywords();
        }

        return $sites;
    }

    function generateNewSelection()
    {
        $c = new Criteria();
        $c->add("status", "validated");
        $c->add($this->getForbiddenRule());
        $c->addOrder("RAND()");
        $c->setLimit(Config::get('selectionSitesCount'));

        $siteIds = $this->site->generateList($c, "siteId");
        $siteIds = array_keys($siteIds);

        $this->setting->set('selectionSiteIds', implode(",", $siteIds));

        $todayDate = date('Y-m-d');
        $this->setting->set('selectionGenerationDate', $todayDate);

        $cache = Cacher::getInstance();
        $cache->clean("tag", array("site", "setting"));

        return $siteIds;
    }

    function getIndexRandomList($refreshNeeded)
    {
        if(!$refreshNeeded)
        {
            $siteIds = explode(",", Config::get("selectionSiteIds"));
        }

        if($refreshNeeded)
        {
            $siteIds = $this->generateNewSelection();
        }

        if(empty($siteIds))return array();

        $c = new Criteria();
        $c->add("siteId", $siteIds, "IN");

        $sites = $this->selectWithNewFlag($c);

        foreach($sites as $key => $site)
        {
            $sites[$key] = $site->toArray();
        }

        return $sites;
    }

    function getNewValidatedSites()
    {
        $c = new Criteria();
        $c->add("status", "validated");
        $c->add($this->getForbiddenRule());
        $c->addOrder("creationDate DESC");
        $c->setLimit(Config::get('maxNewsCount'));

        $fields = "url, siteTitle, siteId, imageSrc, description, creationDate, countryCode";
        return $this->selectWithNewFlag($c, $fields);
    }

    function getValidatedTopVisitedSites()
    {
        $c = new Criteria();
        $c->add("visitsCount", 0, ">");
        $c->add("status", "validated");
        $c->add($this->getForbiddenRule());
        $c->addOrder("visitsCount DESC");
        $c->setLimit(Config::get('maxTopHitsCount'));

        return $this->selectWithNewFlag($c, "siteId, url, imageSrc, siteTitle, description, visitsCount, countryCode");
    }

    function getValidatedTopElectedSites()
    {
        $c = new Criteria();
        $c->add("votesAverage", 0, ">");
        $c->add("status", "validated");
        $c->add($this->getForbiddenRule());
        $c->addOrder("votesAverage DESC");
        $c->addOrder("votesCount DESC");
        $c->setLimit(Config::get('maxTopNotesCount'));

        return $this->selectWithNewFlag($c, "siteId, url, imageSrc, siteTitle, description, votesAverage, votesCount, countryCode");
    }

    function getValidatedTopRankSites($pageRank = false)
    {
        $c = new Criteria();
        $c->add("status", "validated");
        $c->add($this->getForbiddenRule());
        $c->addInnerJoin("cachegoogledetails", "cachegoogledetails.url", "sites.url");
        $c->addOrder("pageRank DESC");
        $c->addOrder("siteId DESC");

        if($pageRank !== false)
        {
            $c->add("pageRank", $pageRank);
        }

        $c->setLimit(Config::get('maxTopRankCount'));

        return $this->selectWithNewFlag($c, "siteId, sites.url, imageSrc, siteTitle, description, pageRank, countryCode");
    }

    function getValidatedTopReferrersSites()
    {
        $c = new Criteria();
        $c->add("referrerTimes", 0, ">");
        $c->add("status", "validated");
        $c->add($this->getForbiddenRule());
        $c->addOrder("referrerTimes DESC");
        $c->setLimit(Config::get('maxTopReferrersCount'));

        return $this->selectWithNewFlag($c, "siteId, sites.url, imageSrc, siteTitle, description, referrerTimes, countryCode");
    }
    
    function getForbiddenRule()
    {
        $prefix = Config::get("DB_PREFIX");
        return "EXISTS(SELECT * FROM ".$prefix."categories WHERE ".$prefix."categories.categoryId=".$prefix."sites.categoryId AND forbidden='0')";
    }

    function getSiteWithDetails($siteId)
    {
        $c = new Criteria();
        $c->add("siteId", $siteId);
        $sites = $this->selectWithNewFlag($c);
        return $sites ? $sites[0] : false;
    }

    function addDefaultSortingOrder($c)
    {
        $order = Config::get("sitesSortyBy");
        if(!in_array($order, array("creationDate", "referrerTimes", "siteTitle")))$order = "creationDate";

        if($order == "referrerTimes")$order .= " DESC";
        $c->addOrder($order);
    }

    function getValidatedSitesInCategory($categoryId, $page = false, $perPage = false)
    {
        if($perPage === false)
        {
            $perPage = Config::get("sitesPerPageInCategory");
        }

        $c = new Criteria();
        $c->add("categoryId", $categoryId);
        $c->add("status", "validated");
        $c->addOrder("sites.priority DESC");
        $this->addDefaultSortingOrder($c);

        if($page !== false)$this->addPageCriteria($c, $page, $perPage);

        return $this->selectWithNewFlag($c);
    }

    function getSites($page = false, $perPage = 10)
    {
        $c = new Criteria();
        $c->addOrder("referrerTimes DESC");
        if($page !== false)$this->addPageCriteria($c, $page, $perPage);

        return $this->selectWithNewFlag($c);
    }

    function createSearchCriteria($phrase)
    {
        $rawPhrase = addslashes($phrase);
        $phrase = strtr($phrase, '+-><()~*"', ' ');
        $words = preg_split("#\s+#", $phrase);
        $phrase = "+".implode(" +", $words);
        
        $phrase = addslashes($phrase);
        $c = new Criteria();
        $c->add("MATCH (siteTitle, description)
        AGAINST ('$phrase' IN BOOLEAN MODE)");
        $c->addOr("LOCATE( '$rawPhrase', url ) > 0 ");

        return $c;
    }

    function addPageCriteria($c, $page, $resultsPerPage)
    {
        if(empty($page) || $page < 1)$page = 1;
        $c->setLimit($resultsPerPage);
        $c->setOffset(($page - 1) * $resultsPerPage);
    }

    function search($phrase, $page = false)
    {
        $c = $this->createSearchCriteria($phrase);
        if($page !== false)$this->addPageCriteria($c, $page, Config::get("sitesPerPageInSearch"));
        return $this->selectWithNewFlag($c);
    }

    function searchValidated($phrase, $page = false)
    {
        $c = $this->createSearchCriteria($phrase);
        $c2 = new Criteria();
        $c2->add($c);
        $c2->add("status", "validated");
        $c2->add($this->getForbiddenRule());
        $c2->addOrder("sites.priority DESC");
        $this->addDefaultSortingOrder($c2);
        if($page !== false)$this->addPageCriteria($c2, $page, Config::get("sitesPerPageInSearch"));
        return $this->selectWithNewFlag($c2);
    }
    
    function searchValidatedSite($phrase)
    {
        $c = new Criteria();
        $c->add("status", "validated");
        
        $c2 = $this->createSearchCriteria($phrase);
        $c->add($c2);
        $c->setLimit(1);
        
        return $this->find($c, "url, description");
    }

    function getSearchValidatedCount($phrase)
    {
        $c = $this->createSearchCriteria($phrase);
        $c2 = new Criteria();
        $c2->add($c);
        $c2->add("status", "validated");
        return $this->getCount($c2);
    }

    function getSitesThatOwnAKeyword($keywordId, $page = false)
    {
        $c = new Criteria();
        $c->add("status", "validated");
        $c->addInnerJoin("keywordsofsites", "keywordsofsites.siteId", "sites.siteId");
        $c->add("keywordsofsites.keywordId", $keywordId);
        $c->addOrder("priority DESC");
        $this->addDefaultSortingOrder($c);

        if($page !== false)$this->addPageCriteria($c, $page, Config::get("sitesPerPageInKeywords"));

        return $this->selectWithNewFlag($c);
    }

    function update($data, Criteria $c)
    {
        if(isset($data['address'])
        || isset($data['city'])
        || isset($data['country'])
        )
        {
            $data['_lng'] = "NULL";
            $data['_lat'] = "NULL";
        }
        
        parent::update($data, $c);

        //if status changed must recalculate validated, refused counts
        if(isset($data['status']) || isset($data['categoryId']))
        {
            $this->updateStats();
        }
    }

    function del(Criteria $c, $updateStats = true)
    {
        $condition = $c->getCondition("siteId");

        $nc = new Criteria();
        $nc->addCondition($condition);

        parent::del($c);

        Model::factoryInstance("keywordsOfSite")->del($nc);
        Model::factoryInstance("comment")->del($nc, false);
        Model::factoryInstance("vote")->del($nc);
        Model::factoryInstance("hit")->del($nc);

        if($updateStats)
        {
            $this->updateStats();
        }
    }

    function insert($data)
    {
        parent::insert($data);
        $this->updateStats();
    }

    function updateStats()
    {
        $this->category->updateValidatedSitesCount();
        Cacher::getInstance()->clean("tag", array("site", "category", "keyword"));
    }

    function validate($newSite, $oldSite = null, $validationOptions = array())
    {
        $categoryId = intval($newSite->categoryId);
        $url = $newSite->url;
        $isAdmin = (!empty($validationOptions['admin']));
        $package = (!empty($validationOptions['package'])) ? $validationOptions['package'] : null;

        // required fields validation
        if(empty($newSite->siteTitle) || empty($newSite->url) || empty($newSite->description))
            return 'Please, fill in the fields URL of site, Title and Description.';

        if(empty($categoryId))
            return 'Please, select a category from the dropdown list.';

        // ensure that site is not registered in this category
        if((!$oldSite || $oldSite->categoryId != $categoryId)
           && empty($validationOptions['forceCategoryDuplicate'])
           && $this->whetherSiteIsRegisteredInCategory($url, $categoryId))
            return 'The site is already registered in this category.';

        // check whether the URL is allowed to be offered
        if(!ereg('^http://', $url))
            return 'The URL must start with http://';

        if($this->getIfHostIsBanned(FrontController::getHostNameFromUrl($url), 0))
            return 'This site is not allowed to be offered.';

        if($this->bannedSite->isBanned($url))
            return 'This site is not allowed to be offered.';

        $minSiteDescriptionLength = Config::get("minSiteDescriptionLength");

        if($minSiteDescriptionLength && utf8_strlen($newSite->description) < $minSiteDescriptionLength && !$isAdmin)
        {
            return _t('Description of site must have minimum').' '.$minSiteDescriptionLength.' '._t('characters length.');
        }
        
        $siteDescriptionMaxLength = $package ? $package['siteDescriptionMaxLength'] : Config::get('siteDescriptionMaxLength');
        
        if($siteDescriptionMaxLength && utf8_strlen($newSite->description) > $siteDescriptionMaxLength)
        {
            return _t('Description of site must have maximum').' '.$siteDescriptionMaxLength.' '._t('characters length.');
        }

        if(!$oldSite && !$isAdmin)
        {
            if((integer)($this->getCountOfSiteCopiesInDifferentCategories($url)) >= (integer)(Config::get('maxCategoriesCountPerSite')))
                return 'This site is not allowed to be offered more times.';
        }

        if(Config::get('inscriptionCheckHttpResponseCode') == 1)
        {
            $httpClient = new HttpClient();
            $httpCode = $httpClient->checkResponseCodeOfSite($url);

            if($httpCode != 200)
                return 'This site\' HTTP response code is not 200. The site is not accepted.';
        }

        if(Config::get('backLinkMandatory') && (!$package || $package['backLinkMandatory']) && empty($newSite->returnBond) && !$isAdmin)
        {
            return 'Backlink is mandatory.';
        }

        if(Config::get('countryFlagsEnabled') && empty($newSite->countryCode) && !$isAdmin)
        {
            return 'Country is mandatory.';
        }

        $category = $this->category->findByPk($categoryId);

        if(!$category || ($category->possibleTender != 1 && empty($validationOptions['forcePossibleTender'])))
            return 'Sites cannot be offered for this category.';

        if($this->bannedEmail->isBanned($newSite->webmasterEmail) && !$isAdmin)
            return 'This email is banned.';

        if($this->bannedIp->isBanned(Request::getInstance()->getIp()))
            return 'Your IP is banned.';

        return '';
    }
}

class SiteRecord extends ModelRecord
{
    function updateImageSrc()
    {
        $this->imageSrc = $this->getThumbnailSrc();
    }

    function getKeywordIds()
    {
        return $this->getKeywords(true);
    }

    function getSimilarSites()
    {
        $c = new Criteria();
        $c->add("categoryId", $this->categoryId);
        $c->add("siteId", $this->siteId, "!=");
        $c->add("status", "validated");

        if(Config::get("similarSiteKeywordMatch"))
        {
            if(empty($this->keywords))return array();
            $keywordIds = implode(",", array_keys($this->keywords));
            $prefix = Config::get("DB_PREFIX");

            $c->add("EXISTS(SELECT * FROM ".$prefix."keywordsofsites
                     WHERE ".$prefix."keywordsofsites.siteId=".$prefix."sites.siteId AND keywordId IN ($keywordIds))");
        }

        $c->addOrder("RAND()");
        $c->setLimit(Config::get("randomSitesInDetailsCount"));

        return Model::factoryInstance("site")->findAll($c, "siteId, siteTitle, description");
    }

    function getKeywords($onlyIds = false)
    {
        $c = new Criteria();
        $c->add("siteId", $this->siteId);

        if($onlyIds)
        {
            $field = "keywordId";
        }
        else
        {
            $field = "keywords.keywordId, keyword";
            $c->addInnerJoin("keywords", "keywords.keywordId", "keywordsofsites.keywordId");
        }

        $values = array();

        foreach(Model::factoryInstance("keywordsOfSite")->findAll($c, $field) as $keyword)
        {
            if($onlyIds)
            {
                $values[] = $keyword['keywordId'];
            }
            else
            {
                $values[$keyword['keywordId']] = $keyword['keyword'];
            }
        }

        return $values;
    }

    function downloadAndCacheThumb($forced = false)
    {
        $imageFileName = Config::get("SITES_THUMBS_PATH"). $this->siteId . ".jpg";

        if(!file_exists($imageFileName) || $forced)
        {
            if(!Config::get("thumbsGeneratorUrl"))return false;
            
            $thumbsGeneratorUrl = Config::get("thumbsGeneratorUrl");
            if(strpos($thumbsGeneratorUrl, "[url]") !== false)
                $url = str_replace("[url]", $this->url, $thumbsGeneratorUrl);
            else
                $url = $thumbsGeneratorUrl . $this->url;
            
            $httpClient = new HttpClient();
            $thumbImage = $httpClient->getSiteContent($url, false, true, true);

            if($thumbImage)
            {
                file_put_contents($imageFileName, $thumbImage);
                return true;
            }
            else
            {
                return false;
            }
        }

        return true;
    }
    
    function deleteImage()
    {
        $thumbsPath = Config::get("SITES_THUMBS_PATH");
        
        if(!empty($this->imageSrc))
        {
            $oldFilePath = $thumbsPath . basename($this->imageSrc);

            if(file_exists($oldFilePath))
            {
                unlink($oldFilePath);
            }
            
            $this->imageSrc = "";
        }  
    }
    
    function updateAscreen()
    {
        $thumbsPath = Config::get("SITES_THUMBS_PATH");
                
        $im = @imagecreatefromjpeg(rtrim($this->url,"/")."/ascreen.jpg");
                          
        if(!empty($im) && @imagesx($im) <= 250 && @imagesy($im) <= 200)
        {
            do
            {
                // select new random name
                $imgSrc = rand(100000000000000000, 9999999999999999999). ".jpg";
                $path = $thumbsPath . $imgSrc;
            }while(file_exists($path)); // while this name is already busy
        
            imagejpeg($im, $path, 90);
            
            $this->deleteImage();
            
            $this->imageSrc = $imgSrc; 
            $this->ascreen = 1;
        }
    }

    function getDefaultImageSrc()
    {
        return FrontController::getResourceUrl("/uploads/images_thumbs/default.jpg");
    }

    function getThumbnailSrc()
    {
        $siteId = $this->siteId;

        if($this->imageSrc)
        {
            return FrontController::getResourceUrl("/uploads/images_thumbs/" . $this->imageSrc);
        }
        else
        {
            if(Config::get('cacheSiteImagesEnabled') == 1)
            {
                $fileName = $siteId . '.jpg';
                $filePath = Config::get("SITES_THUMBS_PATH") . $fileName;

                if(file_exists($filePath))
                {
                    return FrontController::getResourceUrl("/uploads/images_thumbs/" . $fileName);
                }
                else
                {
                    return FrontController::getRewrittedUrl("/site/getThumb/".$siteId);
                }
            }
            else
            {
                return Config::get("thumbsGeneratorUrl") . $this->url;
            }

        }

    }
    
    function getGoogleMap()
    { 
        if(!Config::get("googleMapEnabled")
        || empty($this->address)
        || empty($this->city)
        || empty($this->country))return false;

        $googleMap = new GoogleMap();
        $googleMap->setAPIKey(Config::get("googleMapKey"));
        $googleMap->setZoomLevel(Config::get("googleMapZoom"));

        $longAddress = $this->address.", ".$this->city.", ".$this->country;
        
        if(!is_null($this->lat) && !is_null($this->lng))
        {
            $googleMap->addGeoPoint($this->lat, $this->lng, $longAddress);
        }
        else
        {
            $point = $googleMap->addAddress($longAddress);
            
            if($point !== false)
            {
                Model::factoryInstance("site")->updateByPk($point, $this->siteId);
            }
        }
        
        // No valid points for google map then search by zipCode City
        if(!$googleMap->validPointsExists())
        {
            $googleMap->addAddress($this->zipCode.", ".$this->city.", ".$this->country);
        }

        // No valid points for google map then search by country
        if(!$googleMap->validPointsExists())
        {
            $googleMap->addAddress($this->country);
            $googleMap->setZoomLevel(12);
        }

        // If any valid points for google map then display the map
        $map = $googleMap->getScriptCode();
        
        if($googleMap->validPointsExists())
        {
            $map .= $googleMap->getMapCode();
            return $map;
        }
        
        $googleMap->setZoomLevel(Config::get("googleMapZoom"));
        
        if($googleMap->isServiceProblem())
        {
            $map .= $googleMap->getUserSideMapCode($this->address.", ".$this->city.", ".$this->country);
            return $map;
        }
        
        return false;
    }

}

?>
